import { Component, Input, OnDestroy, OnInit } from '@angular/core';
import { MatDialog } from '@angular/material/dialog';
import { Router } from '@angular/router';
import { LangChangeEvent, TranslateService } from '@ngx-translate/core';
import { FileUploader } from 'ng2-file-upload';
import { NgxUiLoaderService } from 'ngx-ui-loader';
import { environment } from 'projects/pastis/src/environments/environment';
import { Subscription } from 'rxjs';
import { NoticeService } from '../../core/services/notice.service';
import { NotificationService } from '../../core/services/notification.service';
import { ProfileService } from '../../core/services/profile.service';
import { ArchivalProfileUnit } from '../../models/archival-profile-unit';
import { FileNode } from '../../models/file-node';
import { Profile } from '../../models/profile';
import { ProfileDescription } from '../../models/profile-description.model';
import { CreateProfileComponent } from '../../profile/create-profile/create-profile.component';
import { CreateNoticeComponent } from '../../user-actions/create-notice/create-notice.component';
import { PastisDialogDataCreate } from '../../user-actions/save-profile/save-profile.component';
import { PastisDialogData } from '../pastis-dialog/classes/pastis-dialog-data';


function constantToTranslate(edit: boolean) {
  if(edit){
    this.popupSaveCreateNoticeTitleDialog = this.translated('PROFILE.POP_UP_SAVE.SAVE_PROFILE.POPUP_CREATE_NOTICE_TITLE_DIALOG_EDIT');
    this.popupSaveCreateNoticeSubTitleDialog = this.translated('PROFILE.POP_UP_SAVE.SAVE_PROFILE.POPUP_CREATE_NOTICE_SUBTITLE_DIALOG_EDIT');
  }else{
    this.popupSaveCreateNoticeTitleDialog = this.translated('PROFILE.POP_UP_SAVE.SAVE_PROFILE.POPUP_CREATE_NOTICE_TITLE_DIALOG');
    this.popupSaveCreateNoticeSubTitleDialog = this.translated('PROFILE.POP_UP_SAVE.SAVE_PROFILE.POPUP_CREATE_NOTICE_SUBTITLE_DIALOG');
  }
  this.popupSaveCancelLabel = this.translated('PROFILE.POP_UP_SAVE.SAVE_PROFILE.POPUP_SAVE_CANCEL_LABEL');
  this.popupSaveTitleDialog = this.translated('PROFILE.POP_UP_SAVE.SAVE_PROFILE.POPUP_SAVE_TITLE_DIALOG');
  this.popupSaveSubTitleDialog = this.translated('PROFILE.POP_UP_SAVE.SAVE_PROFILE.POPUP_SAVE_SUBTITLE_DIALOG');
  this.popupSaveOkLabel = this.translated('PROFILE.POP_UP_SAVE.SAVE_PROFILE.POPUP_SAVE_OK_LABEL');
  this.popupSaveCreateNoticeCancelLabel = this.translated('PROFILE.POP_UP_SAVE.SAVE_PROFILE.POPUP_CREATE_NOTICE_CANCEL_LABEL');
  this.popupSaveCreateNoticeOkLabel = this.translated('PROFILE.POP_UP_SAVE.SAVE_PROFILE.POPUP_CREATE_NOTICE_OK_LABEL');
  this.popupCreationCancelLabel = this.translated('PROFILE.POP_UP_CREATION.POPUP_CREATION_CANCEL_LABEL');
  this.popupCreationTitleDialog = this.translated('PROFILE.POP_UP_CREATION.POPUP_CREATION_TITLE_DIALOG');
  this.popupCreationSubTitleDialog = this.translated('PROFILE.POP_UP_CREATION.POPUP_CREATION_SUBTITLE_DIALOG');
  this.popupCreationOkLabel = this.translated('PROFILE.POP_UP_CREATION.POPUP_CREATION_OK_LABEL');
}
@Component({
  selector: 'pastis-popup-option',
  templateUrl: './pastis-popup-option.component.html',
  styleUrls: ['./pastis-popup-option.component.scss']
})
export class PastisPopupOptionComponent implements OnInit, OnDestroy {

  popupSaveCancelLabel:string;
  popupSaveTitleDialog:string;
  popupSaveSubTitleDialog:string;
  popupSaveOkLabel:string;

  popupSaveCreateNoticeCancelLabel:string;
  popupSaveCreateNoticeTitleDialog :string;
  popupSaveCreateNoticeSubTitleDialog :string;
  popupSaveCreateNoticeOkLabel :string;
  editProfile: boolean;

  subscription1$: Subscription;
  subscription2$: Subscription;
  _uploadProfileSub: Subscription;
  subscriptions: Subscription[] = [];
  isStandalone: boolean = environment.standalone;
  popupCreationCancelLabel: string;
  popupCreationTitleDialog: string;
  popupCreationSubTitleDialog: string;
  popupCreationOkLabel: string;

  donnees:string[];

  data: FileNode[] = [];

  archivalProfileUnit: ArchivalProfileUnit;
  profile: Profile;

  profileDescription: ProfileDescription;

  @Input()
  sedaUrl: string;
  @Input()
  newProfileUrl: string;
  @Input()
  uploader: FileUploader = new FileUploader({url: ""});

  expanded: boolean = false;


  constructor(private router:Router, private profileService: ProfileService,
              public dialog: MatDialog, private noticeService: NoticeService,
              private translateService: TranslateService, private loaderService:NgxUiLoaderService,
              private notificationService: NotificationService) { }

  ngOnInit(): void {
    constantToTranslate.call(this, this.editProfile);
    this.translatedOnChange();
  }

  translatedOnChange(): void {
    this.translateService.onLangChange
      .subscribe((event: LangChangeEvent) => {
        constantToTranslate.call(this);
        console.log(event.lang);
      });
  }

  translated(nameOfFieldToTranslate: string): string {
    return this.translateService.instant(nameOfFieldToTranslate);
  }

  goToSedaView(){
    window.open(this.sedaUrl);
  }

  uploadProfile(files: File[]):void {
    let fileToUpload: File = files[0];

    if (fileToUpload) {
      const formData = new FormData();
      formData.append('file', fileToUpload, fileToUpload.name);
      this._uploadProfileSub = this.profileService.uploadProfile(formData).subscribe( (response: any) => {
        if (response) {
          console.log('File submited! Reponse is : ', response);

          this.router.navigateByUrl(this.newProfileUrl, { state: response });
        }
      });
    }
  }

  changeExpand(){
    this.expanded = !this.expanded;
  }
  async createProfile() {
    this.loaderService.start();
    let dataToSendToPopUp = <PastisDialogData>{};
    dataToSendToPopUp.titleDialog = this.popupCreationTitleDialog;
    dataToSendToPopUp.subTitleDialog = this.popupCreationSubTitleDialog;
    dataToSendToPopUp.width = '800px';
    dataToSendToPopUp.height = '800px';
    dataToSendToPopUp.okLabel = this.popupCreationOkLabel;
    dataToSendToPopUp.cancelLabel = this.popupCreationCancelLabel;
    const dialogRef = this.dialog.open(CreateProfileComponent, {
        width: '800px',
        panelClass: 'pastis-popup-modal-box',
        data: dataToSendToPopUp
      }
    );
    this.subscription2$ =dialogRef.afterClosed().subscribe((result) => {
      if (result.success){
        console.log(result.action + " PA ou PUA ?")
        if(result.action ==='PA' || result.action ==='PUA'){
          let dataToSendToPopUp = <PastisDialogDataCreate>{};
          dataToSendToPopUp.titleDialog = this.popupSaveCreateNoticeTitleDialog;
          dataToSendToPopUp.subTitleDialog = this.popupSaveCreateNoticeSubTitleDialog;
          dataToSendToPopUp.okLabel = this.popupSaveCreateNoticeOkLabel;
          dataToSendToPopUp.cancelLabel = this.popupSaveCreateNoticeCancelLabel;
          dataToSendToPopUp.modeProfile = result.action;
          const dialogRef = this.dialog.open(CreateNoticeComponent, {
              width: '800px',
              panelClass: 'pastis-popup-modal-box',
              data: dataToSendToPopUp
            }
          );
          dialogRef.afterClosed().subscribe((result) => {
            let retour;
            if (result.success) {
              retour = result.data
              if (result.mode === "PUA") {
                console.log("je suis sur un enregistrement d'un PUA")

                let profileDescription = this.noticeService.puaNotice(retour);
                this.profileService.createArchivalUnitProfile(profileDescription).subscribe(() => {
                  this.changeExpand();
                  this.notificationService.showSuccess("La création de notice a bien été effectué")
                  this.profileService.refreshListProfiles();
                  console.log("ok create")
              })

              } else if (result.mode === "PA") {
                console.log(retour.identifier)

                let profile: Profile = this.noticeService.paNotice(retour, true)
                //STEP 1 : Create Notice
                this.profileService.createProfilePa(profile).subscribe(() => {
                  //console.log("ok create" + createdProfile)
                  this.changeExpand();
                  this.notificationService.showSuccess("La création de notice a bien été effectué")
                  this.profileService.refreshListProfiles();
                })

              }
            }
          });
        }
      }
      });
      this.loaderService.stop();
    this.subscriptions.push(this.subscription2$);
  }

  ngOnDestroy(): void {
    if(this._uploadProfileSub){
      this._uploadProfileSub.unsubscribe();
    }
  }

}
